﻿using Celeste;
using Microsoft.Xna.Framework;
using Monocle;
using MonoMod.Utils;
using System;
using System.Collections;
using Celeste.Mod.Entities;
using System.Reflection;

namespace Celeste.Mod.FastPlayerDeadBodyHelper
{
    public class FastPlayerDeadBody : Entity
    {

        public static void Load()
        {
            On.Celeste.PlayerDeadBody.DeathRoutine += DieFast;
            On.Celeste.PlayerDeadBody.Awake += AwakeFast;
            On.Celeste.PlayerDeadBody.Update += UpdateFast;
        }

        public static void Unload()
        {
            On.Celeste.PlayerDeadBody.DeathRoutine -= DieFast;
            On.Celeste.PlayerDeadBody.Awake -= AwakeFast;
            On.Celeste.PlayerDeadBody.Update -= UpdateFast;
        }

        private static MethodInfo End = typeof(PlayerDeadBody).GetMethod("End", BindingFlags.Instance | BindingFlags.NonPublic);
        Sprite boomSprite = new Sprite(GFX.Game, "characters/explosion/");
        Sprite spinSprite = new Sprite(GFX.Game, "characters/player/");

        public static void AwakeFast(On.Celeste.PlayerDeadBody.orig_Awake orig, PlayerDeadBody self, Scene scene)
        {

            orig(self, scene);
            if (!FastPlayerDeadBodyModule.Settings.DisableExplosions)

            {
                DynData<PlayerDeadBody> selfData = new DynData<PlayerDeadBody>(self);
                selfData["exploded"] = false;
                selfData["finished"] = false;
                FastPlayerDeadBody.SpriteCreate(self);
            }

        }

        private static IEnumerator DieFast(On.Celeste.PlayerDeadBody.orig_DeathRoutine orig, PlayerDeadBody self)
        {
            if (!FastPlayerDeadBodyModule.Settings.DisableExplosions)
            {
                DynData<PlayerDeadBody> selfData = new DynData<PlayerDeadBody>(self);
                //FastPlayerDeadBody.SpriteCreate(self);
                Vector2 bounce = selfData.Get<Vector2>("bounce");
                PlayerSprite sprite = selfData.Get<PlayerSprite>("sprite");
                float scale = selfData.Get<float>("scale");
                DeathEffect deathEffect = selfData.Get<DeathEffect>("deathEffect");
                Color initialHairColor = selfData.Get<Color>("initialHairColor");
                VertexLight light = selfData.Get<VertexLight>("light");
                Player player = selfData.Get<Player>("player");
                Level level = self.SceneAs<Level>();

                selfData["boomSprite"] = new Sprite(GFX.Game, "characters/explosion/");
                //boomSprite.Add("boom", "boom", 0.07f);
                int[] frames = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 };
                selfData.Get<Sprite>("boomSprite").Add("boom", "boom", 0.07f, frames);
                self.Add(selfData.Get<Sprite>("boomSprite"));

                if (bounce != Vector2.Zero)
                {
                    Audio.Play("event:/Madeline_Explodes/madeline_oof", self.Position);
                    scale = 1.5f;
                    Celeste.Freeze(0.05f);
                    yield return null;
                    Vector2 from = self.Position;
                    Vector2 to = from + bounce * 64f; //48
                    Tween tween = Tween.Create(Tween.TweenMode.Oneshot, Ease.CubeOut, 0.75f, start: true);
                    self.Add(tween);
                    tween.OnUpdate = delegate (Tween t)
                    {
                        if (!selfData.Get<bool>("exploded"))
                        {
                            self.Position = from + (to - from) * t.Eased;
                        }
                    };
                    yield return tween.Duration * 0.75f;
                    tween.Stop();
                }
                else
                {
                    //selfData.Get<Sprite>("sprite").Stop();
                    //player.Remove(sprite);
                }
                
                //player.Remove(sprite);
                if (!selfData.Get<bool>("exploded"))
                {
                    selfData["exploded"] = true;
                    self.Position += Vector2.UnitY * -5f;
                    level.Displacement.AddBurst(self.Position, 0.3f, 0f, 80f);
                    level.Shake();
                    Input.Rumble(RumbleStrength.Strong, RumbleLength.Long);
                    if (self.HasGolden)
                    {
                        Audio.Play("event:/new_content/char/madeline/death_golden");
                    }

                    Audio.Play("event:/Madeline_Explodes/explosion", self.Position);
                    selfData.Get<Sprite>("boomSprite").Position += new Vector2(-38, -62);
                    selfData.Get<Sprite>("boomSprite").Play("boom");
                }

                deathEffect = new DeathEffect(Color.Transparent, self.Center - self.Position + new Vector2(-10000, 10000));
                deathEffect.OnUpdate = delegate (float f)
                {
                    light.Alpha = 1f - f;
                };
                self.Add(deathEffect);
                yield return deathEffect.Duration * 0.65f;
                if (self.ActionDelay > 0f)
                {
                    yield return self.ActionDelay;
                }

                End.Invoke(self, new object[] { });
            }
            else
            {
                IEnumerator origEnum = orig(self);
                while (origEnum.MoveNext()) yield return origEnum.Current;
            }
        }

        public static void UpdateFast(On.Celeste.PlayerDeadBody.orig_Update orig, PlayerDeadBody self)
        {

            if (Input.MenuConfirm.Pressed && !FastPlayerDeadBodyModule.Settings.DisableExplosions)
            {
                DynData<PlayerDeadBody> selfData = new DynData<PlayerDeadBody>(self);
                if (!selfData.Get<bool>("finished"))
                {
                    if (!selfData.Get<bool>("exploded"))
                    {
                        //FastPlayerDeadBody.SpriteCreate(self);
                        selfData["exploded"] = true;
                        Player player = selfData.Get<Player>("player");
                        Level level = self.SceneAs<Level>();
                        PlayerSprite sprite = selfData.Get<PlayerSprite>("sprite");
                        selfData["boomSprite"] = new Sprite(GFX.Game, "characters/explosion/");
                        //boomSprite.Add("boom", "boom", 0.07f);
                        int[] frames = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 };
                        selfData.Get<Sprite>("boomSprite").Add("boom", "boom", 0.07f, frames);
                        self.Add(selfData.Get<Sprite>("boomSprite"));

                        selfData.Get<Sprite>("sprite").Stop();
                        //player.Remove(sprite);
                        self.Position += Vector2.UnitY * -5f;
                        level.Displacement.AddBurst(self.Position, 0.3f, 0f, 80f);
                        level.Shake();
                        Input.Rumble(RumbleStrength.Strong, RumbleLength.Long);
                        if (self.HasGolden)
                        {
                            Audio.Play("event:/new_content/char/madeline/death_golden");
                        }

                        Audio.Play("event:/Madeline_Explodes/explosion", self.Position);
                        selfData.Get<Sprite>("boomSprite").Position += new Vector2(-38, -62);
                        selfData.Get<Sprite>("boomSprite").Play("boom");

                        selfData["finished"] = true;

                        if (self.DeathAction == null)
                        {
                            self.DeathAction = level.Reload;
                        }
                        level.DoScreenWipe(wipeIn: false, self.DeathAction);
                    }
                }
            }

            orig(self);
        }

        public static void SpriteCreate(PlayerDeadBody self)
        {
            DynData<PlayerDeadBody> selfData = new DynData<PlayerDeadBody>(self);
            if (selfData.Get<Vector2>("bounce") != Vector2.Zero)
            {
                if (selfData.Get<PlayerSprite>("sprite") != null)
                {
                    DynData<Sprite> spriteData = new DynData<Sprite>(selfData.Get<PlayerSprite>("sprite"));
                    spriteData.Get<Sprite.Animation>("currentAnimation").Goto = new Chooser<string>("deadside", 1f);
                    spriteData.Get<Sprite.Animation>("currentAnimation").Goto = new Chooser<string>("deadup", 1f);
                    spriteData.Get<Sprite.Animation>("currentAnimation").Goto = new Chooser<string>("deaddown", 1f);
                    spriteData.Get<Sprite.Animation>("currentAnimation").Delay = 0.04f;
                }

            }
        }

    }
}
